@@ -19,12 +19,15 @@
 from __future__ import absolute_import, division, print_function
 
 import difflib
+import errno
 import itertools
 import json
 import struct
+import os.path
 import re
 import requests
 import unicodedata
+from unidecode import unidecode
 import warnings
 import six
 from six.moves import urllib
@@ -74,6 +77,41 @@ URL_CHARACTERS = {
 }
 USER_AGENT = 'beets/{}'.format(beets.__version__)
 
+# The content for the base index.rst generated in ReST mode.
+REST_INDEX_TEMPLATE = u'''Lyrics
+======
+
+* :ref:`Song index <genindex>`
+* :ref:`search`
+
+Artist index:
+
+.. toctree::
+   :maxdepth: 1
+   :glob:
+
+   artists/*
+'''
+
+# The content for the base conf.py generated.
+REST_CONF_TEMPLATE = u'''# -*- coding: utf-8 -*-
+master_doc = 'index'
+project = u'Lyrics'
+copyright = u'none'
+author = u'Various Authors'
+latex_documents = [
+    (master_doc, 'Lyrics.tex', project,
+     author, 'manual'),
+]
+epub_title = project
+epub_author = author
+epub_publisher = author
+epub_copyright = copyright
+epub_exclude_files = ['search.html']
+epub_tocdepth = 1
+epub_tocdup = False
+'''
+
 
 # Utilities.
 
@@ -189,6 +227,24 @@ def search_pairs(item):
     return itertools.product(artists, multi_titles)
 
 
+def slug(text):
+    """Make a URL-safe, human-readable version of the given text
+
+    This will do the following:
+
+    1. decode unicode characters into ASCII
+    2. shift everything to lowercase
+    3. strip whitespace
+    4. replace other non-word characters with dashes
+    5. strip extra dashes
+
+    This somewhat duplicates the :func:`Google.slugify` function but
+    slugify is not as generic as this one, which can be reused
+    elsewhere.
+    """
+    return re.sub(r'\W+', '-', unidecode(text).lower().strip()).strip('-')
+
+
 class Backend(object):
     def __init__(self, config, log):
         self._log = log
@@ -263,13 +319,29 @@ class MusiXmatch(SymbolsReplaced):
         html = self.fetch_url(url)
         if not html:
             return
+        if "We detected that your IP is blocked" in html:
+            self._log.warning(u'we are blocked at MusixMatch: url %s failed'
+                              % url)
+            return
         html_part = html.split('<p class="mxm-lyrics__content')[-1]
         lyrics = extract_text_between(html_part, '>', '</p>')
-        return lyrics.strip(',"').replace('\\n', '\n')
+        lyrics = lyrics.strip(',"').replace('\\n', '\n')
+        # another odd case: sometimes only that string remains, for
+        # missing songs. this seems to happen after being blocked
+        # above, when filling in the CAPTCHA.
+        if "Instant lyrics for all your music." in lyrics:
+            return
+        return lyrics
 
 
 class Genius(Backend):
-    """Fetch lyrics from Genius via genius-api."""
+    """Fetch lyrics from Genius via genius-api.
+
+    Simply adapted from
+    bigishdata.com/2016/09/27/getting-song-lyrics-from-geniuss-api-scraping/
+    """
+
+    base_url = "https://api.genius.com"
 
     def __init__(self, config, log):
         super(Genius, self).__init__(config, log)
@@ -279,85 +351,41 @@ class Genius(Backend):
             'User-Agent': USER_AGENT,
         }
 
-    def search_genius(self, artist, title):
-        query = u"%s %s" % (artist, title)
-        url = u'https://api.genius.com/search?q=%s' \
-            % (urllib.parse.quote(query.encode('utf-8')))
+    def lyrics_from_song_api_path(self, song_api_path):
+        song_url = self.base_url + song_api_path
+        response = requests.get(song_url, headers=self.headers)
+        json = response.json()
+        path = json["response"]["song"]["path"]
 
-        self._log.debug(u'genius: requesting search {}', url)
-        try:
-            req = requests.get(
-                url,
-                headers=self.headers,
-                allow_redirects=True
-            )
-            req.raise_for_status()
-        except requests.RequestException as exc:
-            self._log.debug(u'genius: request error: {}', exc)
-            return None
+        # Gotta go regular html scraping... come on Genius.
+        page_url = "https://genius.com" + path
+        page = requests.get(page_url)
+        html = BeautifulSoup(page.text, "html.parser")
 
-        try:
-            return req.json()
-        except ValueError:
-            self._log.debug(u'genius: invalid response: {}', req.text)
-            return None
+        # Remove script tags that they put in the middle of the lyrics.
+        [h.extract() for h in html('script')]
 
-    def get_lyrics(self, link):
-        url = u'http://genius-api.com/api/lyricsInfo'
+        # At least Genius is nice and has a tag called 'lyrics'!
+        # Updated css where the lyrics are based in HTML.
+        lyrics = html.find("div", class_="lyrics").get_text()
 
-        self._log.debug(u'genius: requesting lyrics for link {}', link)
-        try:
-            req = requests.post(
-                url,
-                data={'link': link},
-                headers=self.headers,
-                allow_redirects=True
-            )
-            req.raise_for_status()
-        except requests.RequestException as exc:
-            self._log.debug(u'genius: request error: {}', exc)
-            return None
-
-        try:
-            return req.json()
-        except ValueError:
-            self._log.debug(u'genius: invalid response: {}', req.text)
-            return None
-
-    def build_lyric_string(self, lyrics):
-        if 'lyrics' not in lyrics:
-            return
-        sections = lyrics['lyrics']['sections']
-
-        lyrics_list = []
-        for section in sections:
-            lyrics_list.append(section['name'])
-            lyrics_list.append('\n')
-            for verse in section['verses']:
-                if 'content' in verse:
-                    lyrics_list.append(verse['content'])
-
-        return ''.join(lyrics_list)
+        return lyrics
 
     def fetch(self, artist, title):
-        search_data = self.search_genius(artist, title)
-        if not search_data:
-            return
+        search_url = self.base_url + "/search"
+        data = {'q': title}
+        response = requests.get(search_url, data=data, headers=self.headers)
+        json = response.json()
 
-        if not search_data['meta']['status'] == 200:
-            return
-        else:
-            records = search_data['response']['hits']
-            if not records:
-                return
+        song_info = None
+        for hit in json["response"]["hits"]:
+            if hit["result"]["primary_artist"]["name"] == artist:
+                song_info = hit
+                break
 
-            record_url = records[0]['result']['url']
-            lyric_data = self.get_lyrics(record_url)
-            if not lyric_data:
-                return
-            lyrics = self.build_lyric_string(lyric_data)
-
-            return lyrics
+        if song_info:
+            song_api_path = song_info["result"]["api_path"]
+            return self.lyrics_from_song_api_path(song_api_path)
 
 
 class LyricsWiki(SymbolsReplaced):
@@ -572,7 +600,7 @@ class Google(Backend):
 
 
 class LyricsPlugin(plugins.BeetsPlugin):
-    SOURCES = ['google', 'lyricwiki', 'musixmatch']
+    SOURCES = ['google', 'lyricwiki', 'musixmatch', 'genius']
     SOURCE_BACKENDS = {
         'google': Google,
         'lyricwiki': LyricsWiki,
@@ -595,6 +623,7 @@ class LyricsPlugin(plugins.BeetsPlugin):
                 "76V-uFL5jks5dNvcGCdarqFjDhP9c",
             'fallback': None,
             'force': False,
+            'local': False,
             'sources': self.SOURCES,
         })
         self.config['bing_client_secret'].redact = True
@@ -602,6 +631,15 @@ class LyricsPlugin(plugins.BeetsPlugin):
         self.config['google_engine_ID'].redact = True
         self.config['genius_api_key'].redact = True
 
+        # State information for the ReST writer.
+        # First, the current artist we're writing.
+        self.artist = u'Unknown artist'
+        # The current album: False means no album yet.
+        self.album = False
+        # The current rest file content. None means the file is not
+        # open yet.
+        self.rest = None
+
         available_sources = list(self.SOURCES)
         sources = plugins.sanitize_choices(
             self.config['sources'].as_str_seq(), available_sources)
@@ -658,27 +696,106 @@ class LyricsPlugin(plugins.BeetsPlugin):
             action='store_true', default=False,
             help=u'print lyrics to console',
         )
+        cmd.parser.add_option(
+            u'-r', u'--write-rest', dest='writerest',
+            action='store', default='.', metavar='dir',
+            help=u'write lyrics to given directory as ReST files',
+        )
         cmd.parser.add_option(
             u'-f', u'--force', dest='force_refetch',
             action='store_true', default=False,
             help=u'always re-download lyrics',
         )
+        cmd.parser.add_option(
+            u'-l', u'--local', dest='local_only',
+            action='store_true', default=False,
+            help=u'do not fetch missing lyrics',
+        )
 
         def func(lib, opts, args):
             # The "write to files" option corresponds to the
             # import_write config value.
             write = ui.should_write()
+            if opts.writerest:
+                self.writerest_indexes(opts.writerest)
             for item in lib.items(ui.decargs(args)):
-                self.fetch_item_lyrics(
-                    lib, item, write,
-                    opts.force_refetch or self.config['force'],
-                )
-                if opts.printlyr and item.lyrics:
-                    ui.print_(item.lyrics)
-
+                if not opts.local_only and not self.config['local']:
+                    self.fetch_item_lyrics(
+                        lib, item, write,
+                        opts.force_refetch or self.config['force'],
+                    )
+                if item.lyrics:
+                    if opts.printlyr:
+                        ui.print_(item.lyrics)
+                    if opts.writerest:
+                        self.writerest(opts.writerest, item)
+            if opts.writerest:
+                # flush last artist
+                self.writerest(opts.writerest, None)
+                ui.print_(u'ReST files generated. to build, use one of:')
+                ui.print_(u'  sphinx-build -b html %s _build/html'
+                          % opts.writerest)
+                ui.print_(u'  sphinx-build -b epub %s _build/epub'
+                          % opts.writerest)
+                ui.print_((u'  sphinx-build -b latex %s _build/latex '
+                           u'&& make -C _build/latex all-pdf')
+                          % opts.writerest)
         cmd.func = func
         return [cmd]
 
+    def writerest(self, directory, item):
+        """Write the item to an ReST file
+
+        This will keep state (in the `rest` variable) in order to avoid
+        writing continuously to the same files.
+        """
+
+        if item is None or slug(self.artist) != slug(item.artist):
+            if self.rest is not None:
+                path = os.path.join(directory, 'artists',
+                                    slug(self.artist) + u'.rst')
+                with open(path, 'wb') as output:
+                    output.write(self.rest.encode('utf-8'))
+                self.rest = None
+                if item is None:
+                    return
+            self.artist = item.artist.strip()
+            self.rest = u"%s\n%s\n\n.. contents::\n   :local:\n\n" \
+                        % (self.artist,
+                           u'=' * len(self.artist))
+        if self.album != item.album:
+            tmpalbum = self.album = item.album.strip()
+            if self.album == '':
+                tmpalbum = u'Unknown album'
+            self.rest += u"%s\n%s\n\n" % (tmpalbum, u'-' * len(tmpalbum))
+        title_str = u":index:`%s`" % item.title.strip()
+        block = u'| ' + item.lyrics.replace(u'\n', u'\n| ')
+        self.rest += u"%s\n%s\n\n%s\n\n" % (title_str,
+                                            u'~' * len(title_str),
+                                            block)
+
+    def writerest_indexes(self, directory):
+        """Write conf.py and index.rst files necessary for Sphinx
+
+        We write minimal configurations that are necessary for Sphinx
+        to operate. We do not overwrite existing files so that
+        customizations are respected."""
+        try:
+            os.makedirs(os.path.join(directory, 'artists'))
+        except OSError as e:
+            if e.errno == errno.EEXIST:
+                pass
+            else:
+                raise
+        indexfile = os.path.join(directory, 'index.rst')
+        if not os.path.exists(indexfile):
+            with open(indexfile, 'w') as output:
+                output.write(REST_INDEX_TEMPLATE)
+        conffile = os.path.join(directory, 'conf.py')
+        if not os.path.exists(conffile):
+            with open(conffile, 'w') as output:
+                output.write(REST_CONF_TEMPLATE)
+
     def imported(self, session, task):
         """Import hook for fetching lyrics automatically.
         """
