@@ -24,6 +24,7 @@ import musicbrainzngs
 import re
 
 SUBMISSION_CHUNK_SIZE = 200
+FETCH_CHUNK_SIZE = 100
 UUID_REGEX = r'^[a-f0-9]{8}(-[a-f0-9]{4}){3}-[a-f0-9]{12}$'
 
 
@@ -60,41 +61,90 @@ class MusicBrainzCollectionPlugin(BeetsPlugin):
             config['musicbrainz']['user'].as_str(),
             config['musicbrainz']['pass'].as_str(),
         )
-        self.config.add({'auto': False})
+        self.config.add({
+            'auto': False,
+            'collection': u'',
+            'remove': False,
+        })
         if self.config['auto']:
             self.import_stages = [self.imported]
 
+    def _get_collection(self):
+        collections = mb_call(musicbrainzngs.get_collections)
+        if not collections['collection-list']:
+            raise ui.UserError(u'no collections exist for user')
+
+        # Get all collection IDs, avoiding event collections
+        collection_ids = [x['id'] for x in collections['collection-list']]
+        if not collection_ids:
+            raise ui.UserError(u'No collection found.')
+
+        # Check that the collection exists so we can present a nice error
+        collection = self.config['collection'].as_str()
+        if collection:
+            if collection not in collection_ids:
+                raise ui.UserError(u'invalid collection ID: {}'
+                                   .format(collection))
+            return collection
+
+        # No specified collection. Just return the first collection ID
+        return collection_ids[0]
+
+    def _get_albums_in_collection(self, id):
+        def _fetch(offset):
+            res = mb_call(
+                musicbrainzngs.get_releases_in_collection,
+                id,
+                limit=FETCH_CHUNK_SIZE,
+                offset=offset
+            )['collection']
+            return [x['id'] for x in res['release-list']], res['release-count']
+
+        offset = 0
+        albums_in_collection, release_count = _fetch(offset)
+        for i in range(0, release_count, FETCH_CHUNK_SIZE):
+            offset += FETCH_CHUNK_SIZE
+            albums_in_collection += _fetch(offset)[0]
+
+        return albums_in_collection
+
     def commands(self):
         mbupdate = Subcommand('mbupdate',
                               help=u'Update MusicBrainz collection')
+        mbupdate.parser.add_option('-r', '--remove',
+                                   action='store_true',
+                                   default=None,
+                                   dest='remove',
+                                   help='Remove albums not in beets library')
         mbupdate.func = self.update_collection
         return [mbupdate]
 
+    def remove_missing(self, collection_id, lib_albums):
+        lib_ids = set([x.mb_albumid for x in lib_albums])
+        albums_in_collection = self._get_albums_in_collection(collection_id)
+        remove_me = list(lib_ids - set(albums_in_collection))
+        for i in range(0, len(remove_me), FETCH_CHUNK_SIZE):
+            chunk = remove_me[i:i + FETCH_CHUNK_SIZE]
+            mb_call(
+                musicbrainzngs.remove_releases_from_collection,
+                collection_id, chunk
+            )
+
     def update_collection(self, lib, opts, args):
-        self.update_album_list(lib.albums())
+        self.config.set_args(opts)
+        remove_missing = self.config['remove'].get(bool)
+        self.update_album_list(lib, lib.albums(), remove_missing)
 
     def imported(self, session, task):
         """Add each imported album to the collection.
         """
         if task.is_album:
-            self.update_album_list([task.album])
+            self.update_album_list(session.lib, [task.album])
 
-    def update_album_list(self, album_list):
+    def update_album_list(self, lib, album_list, remove_missing=False):
         """Update the MusicBrainz collection from a list of Beets albums
         """
-        # Get the available collections.
-        collections = mb_call(musicbrainzngs.get_collections)
-        if not collections['collection-list']:
-            raise ui.UserError(u'no collections exist for user')
-
-        # Get the first release collection. MusicBrainz also has event
-        # collections, so we need to avoid adding to those.
-        for collection in collections['collection-list']:
-            if 'release-count' in collection:
-                collection_id = collection['id']
-                break
-        else:
-            raise ui.UserError(u'No collection found.')
+        collection_id = self._get_collection()
 
         # Get a list of all the album IDs.
         album_ids = []
@@ -111,4 +161,6 @@ class MusicBrainzCollectionPlugin(BeetsPlugin):
             u'Updating MusicBrainz collection {0}...', collection_id
         )
         submit_albums(collection_id, album_ids)
+        if remove_missing:
+            self.remove_missing(collection_id, lib.albums())
         self._log.info(u'...MusicBrainz collection updated.')
