@@ -606,6 +606,10 @@ class GStreamerBackend(Backend):
         self._decbin.sync_state_with_parent()
         self._decbin.get_state(self.Gst.CLOCK_TIME_NONE)
 
+        self._decbin.link(self._conv)
+        self._pipe.set_state(self.Gst.State.READY)
+        self._pipe.set_state(self.Gst.State.PLAYING)
+
         return True
 
     def _set_next_file(self):
@@ -855,7 +859,7 @@ class ReplayGainPlugin(BeetsPlugin):
         # value.
         return self.overwrite or \
             any([self.should_use_r128(item) and
-                (not item.r128_item_gain or not item.r128_album_gain)
+                (not item.r128_track_gain or not item.r128_album_gain)
                 for item in album.items()]) or \
             any([not self.should_use_r128(item) and
                 (not item.rg_album_gain or not item.rg_album_peak)
@@ -903,11 +907,11 @@ class ReplayGainPlugin(BeetsPlugin):
 
         self._log.info(u'analyzing {0}', album)
 
-        if (any([self.should_use_r128(item) for item in album.items()]) and
+        if (any([self.should_use_r128(item) for item in album.items()]) and not
                 all(([self.should_use_r128(item) for item in album.items()]))):
                 raise ReplayGainError(
                     u"Mix of ReplayGain and EBU R128 detected"
-                    u"for some tracks in album {0}".format(album)
+                    u" for some tracks in album {0}".format(album)
                 )
 
         if any([self.should_use_r128(item) for item in album.items()]):
